@@ -14,11 +14,15 @@ MySQL backend for WWW:Session
 
 =head1 VERSION
 
-Version 0.10
+Version 0.11
 
 =cut
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
+
+#Internal variable that controls the expired session cleanup process
+#We do a cleanup not faster than every 10 minutes, when we try and retrieve
+my $last_cleanup = 0;
 
 
 =head1 SYNOPSIS
@@ -137,6 +141,12 @@ the string containing the serialized data
 =cut
 sub retrieve {
     my ($self,$sid) = @_;
+
+    if ( $last_cleanup + 600 < time() ) {
+        $last_cleanup = time();
+        my $del_sth = $self->{dbh}->prepare(sprintf("DELETE FROM %s WHERE %s < NOW()",$self->{table},$self->{fields}{expires}));
+        $del_sth->execute()
+    }
     
     my $query = sprintf('SELECT %s as sid,%s as data,UNIX_TIMESTAMP(%s) as expires FROM %s WHERE %s=?',
                         @{$self->{fields}}{qw(sid data expires)},
@@ -200,6 +210,15 @@ sub _check_table_structure {
                         exists $table_fields->{$self->{fields}->{data}};
 }
 
+=head2 _reset_last_cleanup
+
+Resets the last DB cleanup timer, forcing all expired sessions to be removed when the next session is retrieved
+
+=cut
+sub _reset_last_cleanup {
+    $last_cleanup = 0;
+}
+
 =head1 AUTHOR
 
 Gligan Calin Horea, C<< <gliganh at gmail.com> >>
